@@ -117,6 +117,24 @@
 #' days that will be used to calculate statistical metrics. Negative values
 #' indicate previous growing season days. This argument overwrites the calculation
 #' limits defined by lower_limit and upper_limit arguments.
+#' @param dc_method a character string to determine the method to detrend climate
+#' (environmental) data.  Possible values are c("Spline", "ModNegExp", "Mean",
+#' "Friedman", "ModHugershoff"). Defaults to "none" (see dplR R package).
+#' @param dc_nyrs a number giving the rigidity of the smoothing spline, defaults
+#' to 0.67 of series length if nyrs is NULL (see dplR R package).
+#' @param dc_f a number between 0 and 1 giving the frequency response or wavelength
+#' cutoff. Defaults to 0.5 (see dplR R package).
+#' @param dc_pos.slope a logical flag. Will allow for a positive slope to be used
+#' in method "ModNegExp" and "ModHugershoff". If FALSE the line will be horizontal
+#' (see dplR R package).
+#' @param dc_constrain.nls a character string which controls the constraints of
+#' the "ModNegExp" model and the "ModHugershoff"  (see dplR R package).
+#' @param dc_span a numeric value controlling method "Friedman", or "cv" (default)
+#' for automatic choice by cross-validation (see dplR R package).
+#' @param dc_bass a numeric value controlling the smoothness of the fitted curve
+#' in method "Friedman" (see dplR R package).
+#' @param dc_difference	a logical flag. Compute residuals by substraction if TRUE,
+#' otherwise use division (see dplR R package).
 #'
 #' @return a list with 15 elements:
 #' \enumerate{
@@ -166,7 +184,8 @@
 #'                           tidy_env_data_primary = FALSE,
 #'                           previous_year = FALSE, boot = TRUE,
 #'                           tidy_env_data_control = TRUE, boot_n = 10,
-#'                           reference_window = "end")
+#'                           reference_window = "end",
+#'                           day_interval = c(-100, 250))
 #' summary(example_basic)
 #' plot(example_basic, type = 1)
 #' plot(example_basic, type = 2)
@@ -188,6 +207,7 @@
 #'                           tidy_env_data_primary = FALSE,
 #'                           tidy_env_data_control = TRUE,
 #'                           reference_window = "end")
+#'
 #' summary(example_fixed_width)
 #' plot(example_fixed_width, type = 1)
 #' plot(example_fixed_width, type = 2)
@@ -217,8 +237,15 @@ daily_response_seascorr <- function(response, env_data_primary, env_data_control
                            boot_ci_type = "norm", boot_conf_int = 0.95,
                            day_interval = ifelse(c(previous_year == TRUE,
                                                    previous_year == TRUE),
-                                                 c(-1, 366), c(1, 366))
-                           ) {
+                                                 c(-1, 366), c(1, 366)),
+                           dc_method = NULL,
+                           dc_nyrs = NULL,
+                           dc_f = 0.5,
+                           dc_pos.slope = FALSE,
+                           dc_constrain.nls = c("never", "when.fail", "always"),
+                           dc_span = "cv",
+                           dc_bass = 0,
+                           dc_difference = FALSE) {
 
   ##############################################################################
   # 1 day interval is organized
@@ -470,6 +497,17 @@ daily_response_seascorr <- function(response, env_data_primary, env_data_control
 
   }
 
+  # Make sure the selected method is appropriate
+  if (!is.null(dc_method)){
+
+    if (!(dc_method %in% c("Spline", "ModNegExp", "Mean", "Friedman", "ModHugershoff"))){
+
+      stop(paste0('dc_method should be one of "Spline", "ModNegExp", "Mean", "Friedman", "ModHugershoff",
+         but instead it is:',dc_method))
+
+    }
+  }
+
   # Data manipulation
   # If use.previous == TRUE, env_data_primary data has to be rearranged accordingly
   if (previous_year == TRUE) {
@@ -623,6 +661,16 @@ daily_response_seascorr <- function(response, env_data_primary, env_data_control
     env_data_control <- subset(env_data_control, row.names(env_data_control) %in% subset_seq)
     }
 
+  # NA values are not allowed and must be removed from response data.frame
+  if (sum(is.na(response)) > 0){
+
+    prob_year <- row.names(response[is.na(response), , drop = F])
+
+    stop(paste0("NA is not allowed in response data frame. ",
+                "Problematic year is ", prob_year))
+
+  }
+
   # PART 2 - Based on the selected function arguments, different chunks of code
   # will be used. For demonstration:
   # A) Chunks are used if fixed.withd != 0
@@ -683,7 +731,7 @@ daily_response_seascorr <- function(response, env_data_primary, env_data_control
                                  (1 + j): (j + fixed_width)]), na.rm = TRUE)
         } else {
 
-          stop(paste0("aggregate function for is env_data_primary", aggregate_function_env_data_primary, ". Instead it should be mean, median or sum."))
+          stop(paste0("aggregate function for env_data_primary is ", aggregate_function_env_data_primary, ". Instead it should be mean, median or sum."))
 
         }
 
@@ -706,14 +754,35 @@ daily_response_seascorr <- function(response, env_data_primary, env_data_control
         }
 
 
-        x1 <- matrix(x1, nrow = nrow(env_data_primary), ncol = 1)
-        x2 <- matrix(x2, nrow = nrow(env_data_control), ncol = 1)
+        if (!is.null(dc_method)){
+
+          x1 <- dplR::detrend(data.frame(x1), method = dc_method, nyrs = dc_nyrs, f = dc_f,
+                              pos.slope = dc_pos.slope, constrain.nls = dc_constrain.nls,
+                              span = dc_span, bass = dc_bass,  difference = dc_difference)
+
+        } else {
+
+          x1 <- matrix(x1, nrow = nrow(env_data_primary), ncol = 1)
+
+        }
+
+
+        if (!is.null(dc_method)){
+
+          x2 <- dplR::detrend(data.frame(x2), method = dc_method, nyrs = dc_nyrs, f = dc_f,
+                              pos.slope = dc_pos.slope, constrain.nls = dc_constrain.nls,
+                              span = dc_span, bass = dc_bass,  difference = dc_difference)
+
+        } else {
+
+          x2 <- matrix(x2, nrow = nrow(env_data_primary), ncol = 1)
+
+        }
 
         my_temporal_data <- cbind(response[, 1], x1, x2)
         colnames(my_temporal_data) <- c("x", "y", "z")
 
 
-
         if (boot == FALSE){
 
           temporal_correlation <- partial.r(data=my_temporal_data, x=c("x","y"), y="z", use="pairwise",method = pcor_method)[2]
@@ -926,13 +995,34 @@ daily_response_seascorr <- function(response, env_data_primary, env_data_control
 
       } else {
 
-        stop(paste0("aggregate function is ", aggregate_function_env_data_control, ". Instead it should be mean, median or sum."))
+        stop(paste0("aggregate function for env_data_control is ", aggregate_function_env_data_control, ". Instead it should be mean, median or sum."))
 
       }
 
-      x1 <- matrix(x1, nrow = nrow(env_data_primary), ncol = 1)
+      if (!is.null(dc_method)){
 
-      x2 <- matrix(x2, nrow = nrow(env_data_primary), ncol = 1)
+        x1 <- dplR::detrend(data.frame(x1), method = dc_method, nyrs = dc_nyrs, f = dc_f,
+                            pos.slope = dc_pos.slope, constrain.nls = dc_constrain.nls,
+                            span = dc_span, bass = dc_bass,  difference = dc_difference)
+
+      } else {
+
+        x1 <- matrix(x1, nrow = nrow(env_data_primary), ncol = 1)
+
+      }
+
+
+      if (!is.null(dc_method)){
+
+        x2 <- dplR::detrend(data.frame(x2), method = dc_method, nyrs = dc_nyrs, f = dc_f,
+                            pos.slope = dc_pos.slope, constrain.nls = dc_constrain.nls,
+                            span = dc_span, bass = dc_bass,  difference = dc_difference)
+
+      } else {
+
+        x2 <- matrix(x2, nrow = nrow(env_data_primary), ncol = 1)
+
+      }
 
 
       my_temporal_data <- cbind(response[, 1], x1, x2)
@@ -1175,15 +1265,33 @@ daily_response_seascorr <- function(response, env_data_primary, env_data_control
                                                            (as.numeric(plot_column) +
                                                               as.numeric(row_index) - 1)]),1 , sum, na.rm = TRUE))
   } else if (aggregate_function_env_data_primary == 'mean'){
-    x1_original <- data.frame(data.frame(rowMeans(env_data_primary_original[, as.numeric(plot_column):
-                                            (as.numeric(plot_column) +
-                                               as.numeric(row_index) - 1)]),
-                                 na.rm = TRUE))
+
+     x1_original <- data.frame(apply(data.frame(env_data_primary_original[, as.numeric(plot_column):
+                                                                           (as.numeric(plot_column) +
+                                                                              as.numeric(row_index) - 1)]),1 , mean, na.rm = TRUE))
   } else {
-    stop(paste0("aggregate function is ", aggregate_function_env_data_primary, ". Instead it should be mean, median or sum."))
+    stop(paste0("aggregate function for env_data_primary is ", aggregate_function_env_data_primary, ". Instead it should be mean, median or sum."))
   }
 
 
+  if (aggregate_function_env_data_control == 'median'){
+    x2_original <- data.frame(apply(data.frame(env_data_control_original[, as.numeric(plot_column):
+                                                                           (as.numeric(plot_column) +
+                                                                              as.numeric(row_index) - 1)]),1 , median, na.rm = TRUE))
+  } else if (aggregate_function_env_data_control == 'sum'){
+    x2_original <- data.frame(apply(data.frame(env_data_control_original[, as.numeric(plot_column):
+                                                                           (as.numeric(plot_column) +
+                                                                              as.numeric(row_index) - 1)]),1 , sum, na.rm = TRUE))
+  } else if (aggregate_function_env_data_control == 'mean'){
+
+    x2_original <- data.frame(apply(data.frame(env_data_control_original[, as.numeric(plot_column):
+                                                                           (as.numeric(plot_column) +
+                                                                              as.numeric(row_index) - 1)]),1 , mean, na.rm = TRUE))
+
+      } else {
+    stop(paste0("aggregate function for env_data_control is ", aggregate_function_env_data_control, ". Instead it should be mean, median or sum."))
+  }
+
 }
 
 
@@ -1202,7 +1310,7 @@ daily_response_seascorr <- function(response, env_data_primary, env_data_control
       x1 <- data.frame(apply(data.frame(env_data_primary[, (as.numeric(plot_column) - as.numeric(row_index) + 1):
                                            (as.numeric(plot_column))]),1 , mean, na.rm = TRUE))
     } else {
-      stop(paste0("aggregate function is ", aggregate_function_env_data_primary, ". Instead it should be mean, median or sum."))
+      stop(paste0("aggregate function for env_data_primary is ", aggregate_function_env_data_primary, ". Instead it should be mean, median or sum."))
     }
 
 
@@ -1217,11 +1325,11 @@ daily_response_seascorr <- function(response, env_data_primary, env_data_control
       x2 <- data.frame(apply(data.frame(env_data_control[, (as.numeric(plot_column) - as.numeric(row_index) + 1):
                                                 (as.numeric(plot_column))]),1 , mean, na.rm = TRUE))
     } else {
-      stop(paste0("aggregate function is ", aggregate_function_env_data_control, ". Instead it should be mean, median or sum."))
+
+      stop(paste0("aggregate function for env_data_control is ", aggregate_function_env_data_control, ". Instead it should be mean, median or sum."))
+
     }
 
-
-
     ## Once again, the same procedure, to get the optimal sequence, but this time for whole data, not only
     # for the analysed period.
 
@@ -1235,9 +1343,22 @@ daily_response_seascorr <- function(response, env_data_primary, env_data_control
       x1_original <- data.frame(apply(data.frame(env_data_primary_original[, (as.numeric(plot_column) - (as.numeric(row_index) + 1):
                                                              as.numeric(plot_column))]),1 , mean, na.rm = TRUE))
     } else {
-      stop(paste0("aggregate function is ", aggregate_function_env_data_primary, ". Instead it should be mean, median or sum."))
+      stop(paste0("aggregate function for env_data_primary is ", aggregate_function_env_data_primary, ". Instead it should be mean, median or sum."))
     }
 
+
+    if (aggregate_function_env_data_control == 'median'){
+      x2_original <- data.frame(apply(data.frame(env_data_control_original[, (as.numeric(plot_column) - (as.numeric(row_index) + 1):
+                                                                                as.numeric(plot_column))]),1 , median, na.rm = TRUE))
+    } else if (aggregate_function_env_data_control == 'sum'){
+      x2_original <- data.frame(apply(data.frame(env_data_control_original[, (as.numeric(plot_column) - (as.numeric(row_index) + 1):
+                                                                                as.numeric(plot_column))]),1 , sum, na.rm = TRUE))
+    } else if (aggregate_function_env_data_control == 'mean'){
+      x2_original <- data.frame(apply(data.frame(env_data_control_original[, (as.numeric(plot_column) - (as.numeric(row_index) + 1):
+                                                                                as.numeric(plot_column))]),1 , mean, na.rm = TRUE))
+    } else {
+      stop(paste0("aggregate function for env_data_control is ", aggregate_function_env_data_control, ". Instead it should be mean, median or sum."))
+    }
 }
 
 
@@ -1267,7 +1388,7 @@ daily_response_seascorr <- function(response, env_data_primary, env_data_control
                                            (round2((as.numeric(plot_column) + as.numeric(row_index)/2)) - adjustment_2)]),
                                 1 , mean, na.rm = TRUE))
     } else {
-      stop(paste0("aggregate function is ", aggregate_function_env_data_primary, ". Instead it should be mean, median or sum."))
+      stop(paste0("aggregate function for env_data_primary is ", aggregate_function_env_data_primary, ". Instead it should be mean, median or sum."))
     }
 
 
@@ -1287,7 +1408,7 @@ daily_response_seascorr <- function(response, env_data_primary, env_data_control
                                                 (round2((as.numeric(plot_column) + as.numeric(row_index)/2)) - adjustment_2)]),
                              1 , mean, na.rm = TRUE))
     } else {
-      stop(paste0("aggregate function is ", aggregate_function_env_data_control, ". Instead it should be mean, median or sum."))
+      stop(paste0("aggregate function for env_data_control is ", aggregate_function_env_data_control, ". Instead it should be mean, median or sum."))
     }
 
 
@@ -1307,7 +1428,26 @@ daily_response_seascorr <- function(response, env_data_primary, env_data_control
                                                              (round2((as.numeric(plot_column) + as.numeric(row_index)/2)) - adjustment_2)]),
                                          1 , mean, na.rm = TRUE))
     } else {
-      stop(paste0("aggregate function is ", aggregate_function_env_data_primary, ". Instead it should be mean, median or sum."))
+      stop(paste0("aggregate function for env_data_primary is ", aggregate_function_env_data_primary, ". Instead it should be mean, median or sum."))
+    }
+
+
+    if (aggregate_function_env_data_control == 'median'){
+      x2_original <- data.frame(apply(data.frame(env_data_control_original[, (round2((as.numeric(plot_column) - (as.numeric(row_index))/2)) - adjustment_1):
+                                                                             (round2((as.numeric(plot_column) + as.numeric(row_index)/2)) - adjustment_2)]),
+                                      1 , median, na.rm = TRUE))
+
+    } else if (aggregate_function_env_data_control == 'sum'){
+      x2_original <- data.frame(apply(data.frame(env_data_control_original[, (round2((as.numeric(plot_column) - (as.numeric(row_index))/2)) - adjustment_1):
+                                                                             (round2((as.numeric(plot_column) + as.numeric(row_index)/2)) - adjustment_2)]),
+                                      1 , sum, na.rm = TRUE))
+
+    } else if (aggregate_function_env_data_control == 'mean'){
+      x2_original <- data.frame(apply(data.frame(env_data_control_original[, (round2((as.numeric(plot_column) - (as.numeric(row_index))/2)) - adjustment_1):
+                                                                             (round2((as.numeric(plot_column) + as.numeric(row_index)/2)) - adjustment_2)]),
+                                      1 , mean, na.rm = TRUE))
+    } else {
+      stop(paste0("aggregate function for env_data_control is ", aggregate_function_env_data_control, ". Instead it should be mean, median or sum."))
     }
 
   }
@@ -1315,10 +1455,20 @@ daily_response_seascorr <- function(response, env_data_primary, env_data_control
 
 
 
-
-
 # Naredi conclusion
 
+  if (!is.null(dc_method)){
+
+    x1 <- dplR::detrend(x1, method = dc_method, nyrs = dc_nyrs, f = dc_f,
+                           pos.slope = dc_pos.slope, constrain.nls = dc_constrain.nls,
+                           span = dc_span, bass = dc_bass,  difference = dc_difference)
+
+    x2 <- dplR::detrend(x2, method = dc_method, nyrs = dc_nyrs, f = dc_f,
+                        pos.slope = dc_pos.slope, constrain.nls = dc_constrain.nls,
+                        span = dc_span, bass = dc_bass,  difference = dc_difference)
+  }
+
+
   x1_full <- cbind(response, x1, x2)
   colnames(x1_full)[ncol(x1_full)-1] <- "Optimized_return_primary"
   colnames(x1_full)[ncol(x1_full)] <- "Optimized_return_control"
@@ -1328,14 +1478,32 @@ daily_response_seascorr <- function(response, env_data_primary, env_data_control
 
 
   # original
-  x1_full_original <- x1_original
-  colnames(x1_full_original) <- "Optimized_return"
-  colnames(x1) <- "Optimized.rowNames"
 
+  if (!is.null(dc_method)){
+
+    x1_original <- dplR::detrend(x1_original, method = dc_method, nyrs = dc_nyrs, f = dc_f,
+                        pos.slope = dc_pos.slope, constrain.nls = dc_constrain.nls,
+                        span = dc_span, bass = dc_bass,  difference = dc_difference)
+
+    x2_original <- dplR::detrend(x2_original, method = dc_method, nyrs = dc_nyrs, f = dc_f,
+                                 pos.slope = dc_pos.slope, constrain.nls = dc_constrain.nls,
+                                 span = dc_span, bass = dc_bass,  difference = dc_difference)
+    }
+
+  # x1_full_original <- cbind(x1_original, x2_original)
+  x1_full_original <- merge(x1_original, x2_original, by = 0, all = TRUE)
+  colnames(x1_full_original)[1] <- "year"
+  x1_full_original <- years_to_rownames(x1_full_original, "year")
+  colnames(x1_full_original)[ncol(x1_full_original)-1] <- "Optimized_return_primary"
+  colnames(x1_full_original)[ncol(x1_full_original)] <- "Optimized_return_control"
+
+  # x1_full_original <- x1_original
+  # colnames(x1_full_original) <- "Optimized_return"
+
+  colnames(x1) <- "Optimized.rowNames"
   my_temporal_data <- cbind(x1_full[,1], x1_full[,2], x1_full[,3])
   colnames(my_temporal_data) <- c("x", "y", "z")
   test_calculation <- partial.r(data=my_temporal_data, x=c("x","y"), y="z", use="pairwise",method = pcor_method)[2]
-
   test_logical <- as.numeric(max_calculation) == as.numeric(test_calculation)
 
 # if (test_logical == FALSE){
